# Mode N3: RestAPI

This mode allows you to launch virtual machines and/or containers on a dedicated remote MyBee via RestAPI.
Despite the fact that the API service can be launched on any Linux/BSD system, in this chapter we only consider interaction with the MyBee instance.

## Preparing the remote host

We assume that you have installed the latest available version of [MyBee Appliance](https://myb.convectix.com) on a dedicated bare metal - this can be either a server platform or a regular DesktopPC.
In this mode of operation, accounts/users created in the MyBee system are not used in any way - access and division of VMs into groups is regulated by your SSH public key, so only the pubkey is used.

Log in to the MyBee console using 'root' access via http://<MYBEE_SRV/shell/, or using SSH access.

The first thing you need to decide is how open your MyBee server is.

Initially, MyBee was intended only for 'trusted' environments - for example, for home installation, so everyone who can access it can create virtual machines until the physical server's resources are exhausted. 

This behavior is influenced by two parameters that you choose:

  - Configure Hosts Allow for API (disabled)     - When activated, you can list the IP addresses for which the MyBee API will be opened;
  - Configure Pubkey WhiteList (disabled)        - When activating, you will need to list the pubkeys for which the MyBee API will be opened;

More complex configurations in the form of RBAC/user functionality are achieved by MyBee extensions and will not be discussed here, but we would like to remind you once again that ill-considered 
system settings will give server resources to those for whom they are not intended.

Accordingly, if the above parameters are not in the Disabled state, enter the pubkey from which you plan to create the VM (and, if necessary, enter your client's IP in the white list);

## backend selection

Let's consider two options for working with MyBee via API - when the Configure Hosts Allow for API/Configure Pubkey WhiteList options are disabled (full access is allowed) and when you want to restrict access.

In the first case, everything is ready to work out of the box - just specify the IP/FQDN of the server, using the default key generated by the application:

In MyBee-QT, click the [+] Create button and in the dialog box, use the following elements to configure the connection:

  - 1) Specify the scheme (HTTP) or (HTTPS) by which the MyBee instance is available and its IP address in the 'Address' field (or DNS name, if it has been added to the name service). In our example, the server uses 172.16.0.91;
  - 2) this key is created by the application automatically - even if it is used, nothing needs to be changed;
  - 3) when you click on 'Profile', the application will start interacting with the server and if you have done everything correctly, you will see a list of available profiles for creating virtual environments on the remote server;

![mybqt-http1.png](https://myb.convectix.com/img/mybqt-http1.png?raw=true)

The second case - we will use MyBee ACL and a white list of .pub keys. 
Let's also use the system key, which the application kindly generated for us. 

In the same directory where the private key is located, the application generated its public part - in a file with the .pub extension. 

Open this file in Notepad or view it with the `cat` utility - we need a line from this file:

```
cat ~/.config/mybee-qt/id_ed25519.pub
ssh-ed25519 AAAAC3NzaC1lZDI1NTE5AAAAIO7H/yjTjUBxE08pCXTsYj0blKQq1YJFGjeuR2IvsfIu mybee-qt@local
```

Copy the line `ssh-ed25519 AAAAC3NzaC1lZDI1NTE5AAAAIO7H/yjTjUBxE08pCXTsYj0blKQq1YJFGjeuR2IvsfIu mybee-qt@local` and select 'Configure Pubkey WhiteList' in the MyBee console:

![mybqt-http2.png](https://myb.convectix.com/img/mybqt-http2.png?raw=true)

We confirm that we want to activate the ACL by answering 'yes':
```
Turn ACL ON? ('n' or '0' - leave as disabled) 
[yes(1) or no(0)]
```

and in the opened text editor we insert our entry from the .pub file into the first empty line:

![mybqt-http3.png](https://myb.convectix.com/img/mybqt-http3.png?raw=true)

When ready, press F10 to exit the editor, not forgetting to save the changes:

![mybqt-http4.png](https://myb.convectix.com/img/mybqt-http4.png?raw=true)

Now ACL is enabled and you can work with virtual environments via HTTP(s)/API only using your key.


### Guest OS profile selection

In the next step, select the guest OS type (a grouped list of the main derivatives/families (Linux, BSD, Windows, Other)) and the desired distribution itself.
The list of profiles is updated both when updating the application version and by a separate directive. In addition, you can create your own profiles and even gold images based on existing ones - read about this in the [relevant chapter](profiles.md)

![cbsd_local2.png](https://myb.convectix.com/img/cbsd_local2.png?raw=true)

### VM resources: vCPU, RAM, storage

The last step before creation is to select the guest machine configuration in the form of virtual processor cores, RAM, and system disk space.
By default, you are offered the minimum possible characteristics for a particular profile, which guarantee a normal OS startup, but depending on how you will operate the VM,
you may need additional resources. After creation, you can always reconfigure these parameters without recreating the VM.

![cbsd_local3.png](https://myb.convectix.com/img/cbsd_local3.png?raw=true)

### The process of creating a VM

The user no longer takes part in the further process - if the image has not been previously warmed up (use the selected profile for the first time), the application will scan available mirrors for the selected image to obtain the fastest resource for you and will start downloading the image. The speed of this stage depends mainly on the speed of your Internet - just wait until the process of obtaining the image is complete.

---

:information_source: If you are not satisfied with the speed of the mirrors or have difficulty getting an image, [be sure to read this material](https://github.com/cbsd/mirrors).

---


![cbsd_local4.png](https://myb.convectix.com/img/cbsd_local4.png?raw=true)

The image on the basis of which your VM will be created will be saved and cached, so reusing this profile will lead to faster creation of the VM, since there is no need to download the image.
At the end of the VM creation process, the dialog returns you to the familiar main screen of the application.

![cbsd_local5.png](https://myb.convectix.com/img/cbsd_local5.png?raw=true)

To get started with VMs, you can either go to the [Working with VMs](myb-qt-vm.md) chapter, or read about two other methods of working with hypervisors if you have dedicated servers based on [FreeBSD OS](https://www.freebsd.org/)/[Linux](https://kernel.org/) or use the [MyBee](https://myb.convectix.com) distribution.


---

**<<_**__[Prev: Mode N2: via SSH protocol](myb-qt-cbsd-ssh.md)__ $~~~$ | $~~~$ __[Next:]()__**_>>**
